/*
 * Copyright 2002 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _HONEYD_H_
#define _HONEYD_H_

#define PIDFILE			"/var/run/honeyd.pid"

#define TCP_DEFAULT_SIZE	128
#define TCP_MAX_SIZE		4096
#define TCP_MAX_SEND		512

#define HONEYD_CLOSE_WAIT	10
#define HONEYD_SYN_WAIT		20
#define HONEYD_MAX_ACTIVE	120
#define HONEYD_DFL_TTL		64

#define HONEYD_POLL_INTERVAL	{0, 10000}

struct delay {
	struct event timeout;

	int ttl;
	struct addr host;
	struct template *tmpl;
};

enum status {PORT_OPEN = 0, PORT_PROXY, PORT_BLOCK, PORT_RESET};

struct action {
	char *action;
	struct addrinfo *aitop;
	enum status status;
};

struct port {
	SPLAY_ENTRY(port) node;

	int proto;
	u_short number;

	struct action action;
};

SPLAY_HEAD(porttree, port);

struct personality;
struct template {
	SPLAY_ENTRY(template) node;

	char *name;

	struct porttree ports;

	struct action icmp;
	struct action tcp;
	struct action udp;

	struct personality *person;

	int id;
	uint32_t seq;
	int seqcalls;
	uint32_t timestamp;
	struct timeval tv;

	uint16_t drop_inrate;

	uid_t uid;
	gid_t gid;
};

struct tcp_con {
	SPLAY_ENTRY(tcp_con) node;

	ip_addr_t ip_src;
	ip_addr_t ip_dst;
	uint16_t th_sport;
	uint16_t th_dport;

	uint8_t dupacks;
	uint32_t snd_una;

	uint32_t rcv_next;
	uint32_t last_acked;

	struct template *tmpl;
	uint8_t rcv_flags;

	pid_t pid;
	int pfd;
	u_char *payload;
	u_int psize;
	u_int plen;		/* date in buffer */
	u_int poff;		/* current send offset */

	u_char *readbuf;
	u_int rsize;
	u_int rlen;

	struct event pread;
	struct event pwrite;

	uint8_t state;
	uint8_t sentfin:1,
		finacked:1,
		sawwscale:1,
		sawtimestamp:1,
		fdconnected:1,	     
		unused:3;

	u_short	mss;
	u_short window;
	uint32_t echotimestamp;

	u_short retrans_time;

	struct event timeout;
	struct event retrans_timeout;
};

#define TCP_BYTESINFLIGHT(x)	(x)->poff
#define TCP_MAX_INFLIGHT	4096

void honeyd_dispatch(struct template *, struct ip_hdr *);
char *honeyd_contoa(struct tcp_con *);

void cmd_trigger_read(struct tcp_con *);
void cmd_trigger_write(struct tcp_con *);
void cmd_free(struct tcp_con *);
int cmd_fork(struct tcp_con *, char *, char **);
int cmd_add_readbuf(struct tcp_con *, u_char *, u_int);
void cmd_drain_payload(struct tcp_con *con, u_int len);
struct addrinfo;
struct addrinfo *cmd_proxy_getinfo(char *, short);
int cmd_proxy_connect(struct tcp_con *con, struct addrinfo *);

void tcp_free(struct tcp_con *);
int tcp_send(struct tcp_con *, uint8_t, u_char *, u_int);
void tcp_senddata(struct tcp_con *, uint8_t);
void tcp_sendfin(struct tcp_con *);

void config_init(void);
struct template *template_find(char *);
struct port *port_find(struct template *, int, int);

int parse_configuration(FILE *, char *);


char *strrpl(char *, size_t, char *, char *);
#endif
